/******************************************************************************
 *
 *                Microchip Memory Disk Drive File System
 *
 ******************************************************************************
 * FileName:        HardwareProfile.h
 * Dependencies:    None
 * Processor:       PIC18/PIC24/dsPIC30/dsPIC33/PIC32
 * Compiler:        C18/C30/C32
 * Company:         Microchip Technology, Inc.
 * Version:         1.1.2
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
*****************************************************************************/


#ifndef _HARDWAREPROFILE_H_
#define _HARDWAREPROFILE_H_


// Determines processor type automatically
#ifdef __18CXX
	#define USE_PIC18
#elif defined __PIC24F__
	#define USE_16BIT
	#define USE_PIC24F
#elif defined __PIC24H__
	#define USE_16BIT
	#define USE_PIC24H
#elif defined __dsPIC30F__
	#define USE_16BIT
	#define USE_PIC30
#elif defined __dsPIC33F__
	#define USE_16BIT
	#define USE_PIC33
#elif defined __PIC32MX__
	#define USE_32BIT
	#define USE_PIC32
#else
	#error "Use PIC18, PIC24, dsPIC30, dsPIC33 or PIC32 processor"
#endif


// Define your clock speed here

// Sample clock speed for PIC18
#if defined (__18CXX)

    #define GetSystemClock()        40000000                        // System clock frequency (Hz)
    #define GetPeripheralClock()    GetSystemClock()                // Peripheral clock freq.
    #define GetInstructionClock()   (GetSystemClock() / 4)          // Instruction clock freq.

// Sample clock speed for a 16-bit processor
#elif defined (__C30__)

    #define GetSystemClock()        32000000
    #define GetPeripheralClock()    GetSystemClock()
    #define GetInstructionClock()   (GetSystemClock() / 2)


    // Clock values
    #define MILLISECONDS_PER_TICK       10
    #define TIMER_PRESCALER             TIMER_PRESCALER_8   // 8MHz: TIMER_PRESCALER_1
    #define TIMER_PERIOD                20000                // 10ms=20000, 1ms=2000

// Sample clock speed for a 32-bit processor
#elif defined (__PIC32MX__)

    //#define RUN_AT_48MHZ
    //#define RUN_AT_24MHZ
    #define RUN_AT_60MHZ

    // Various clock values
    #if defined(RUN_AT_48MHZ)
        #define GetSystemClock()            48000000UL
        #define GetPeripheralClock()        48000000UL
        #define GetInstructionClock()       (GetSystemClock())
    #elif defined(RUN_AT_24MHZ)
        #define GetSystemClock()            24000000UL
        #define GetPeripheralClock()        24000000UL
        #define GetInstructionClock()       (GetSystemClock())
    #elif defined(RUN_AT_60MHZ)    
        #define GetSystemClock()            (60000000ul)
        #define GetPeripheralClock()        (GetSystemClock()) 
        #define GetInstructionClock()       (GetSystemClock())
    #else
        #error Choose a speed
    #endif

    // Clock values
    #define MILLISECONDS_PER_TICK       10                  // -0.000% error
    #define TIMER_PRESCALER             TIMER_PRESCALER_8   // At 60MHz
    #define TIMER_PERIOD                37500               // At 60MHz
#endif
    

// Define the baud rate constants
#if defined(__C30__)
    #define BAUDRATE2       57600UL
    #define BRG_DIV2        16
    #define BRGH2           0
#elif defined (__PIC32MX__)
    #define BAUDRATE2       57600UL
    #define BRG_DIV2        16 
    #define BRGH2           0 
#endif



// Select your interface type
// This library currently only supports a single physical interface layer

//HACKADAY:we use a SPI interface. hardware is setup below....
#define USE_SD_INTERFACE_WITH_SPI       // SD-SPI.c and .h
//#define USE_CF_INTERFACE_WITH_PMP       // CF-PMP.c and .h
//#define USE_MANUAL_CF_INTERFACE         // CF-Bit transaction.c and .h
//#define USE_USB_INTERFACE               // USB host MSD library


/*********************************************************************/
/******************* Pin and Register Definitions ********************/
/*********************************************************************/

//HACKADAY:we removed hardware definitions that we don't need to make the file clearer

/* SD Card definitions: Change these to fit your application when using
   an SD-card-based physical layer                                   */

#ifdef USE_SD_INTERFACE_WITH_SPI
    #ifdef USE_PIC18
     
    #elif defined (USE_PIC24F) || defined (USE_PIC24H) //*******  16-bit  *******   

//HACKADAY:	define the SPI interface to use with the SD card
//if you use V1 prototype hardware be sure to check your pin configuration, 
//it will depend on the SD card holder you use.
        #define SD_CS				LATBbits.LATB8
        #define SD_CS_TRIS			TRISBbits.TRISB8
        
        #define SD_CD				PORTBbits.RB12
        #define SD_CD_TRIS			TRISBbits.TRISB12

		//No write enable on uSD hardware
     //   #define SD_WE				PORTBbits.RB0
     //   #define SD_WE_TRIS			TRISBbits.TRISB0
        
        // Registers for the SPI module you want to use
        #define SPICON1				SPI2CON1
        #define SPISTAT				SPI2STAT
        #define SPIBUF				SPI2BUF
        #define SPISTAT_RBF			SPI2STATbits.SPIRBF
        #define SPICON1bits			SPI2CON1bits
        #define SPISTATbits			SPI2STATbits
        #define SPIENABLE           SPI2STATbits.SPIEN

		// SPI SCK, SDI, SDO pins are automatically controlled by the 
		// PIC24/dsPIC/PIC32 SPI module         
		// Tris pins for SCK/SDI/SDO lines
        #define SPICLOCK			TRISBbits.TRISB10
        #define SPIIN				TRISBbits.TRISB11
        #define SPIOUT				TRISBbits.TRISB9

		// Peripheral Pin Select Outputs
		#define NULL_IO		0
		#define C1OUT_IO	1
		#define C2OUT_IO	2
		#define U1TX_IO		3
		#define U1RTS_IO	4
		#define U2TX_IO		5
		#define U2RTS_IO	6
		#define SDO1_IO		7
		#define SCK1OUT_IO	8
		#define SS1OUT_IO	9
		#define SDO2_IO		10
		#define SCK2OUT_IO	11
		#define SS2OUT_IO	12
		#define OC1_IO		18
		#define OC2_IO		19
		#define OC3_IO		20
		#define OC4_IO		21
		#define OC5_IO		22

        // Will generate an error if the clock speed is too low to interface to the card
        #if (GetSystemClock() < 100000)
            #error Clock speed must exceed 100 kHz
        #endif    
    
    #elif defined (USE_PIC32) //*******  32-bit  *******
     
    #endif

#endif
// Macros for input and output TRIS bits
#define INPUT_PIN   1
#define OUTPUT_PIN  0



#ifdef USE_PIC18
    #include <p18cxxx.h>
#elif defined USE_PIC24F
    #include <p24fxxxx.h>
#elif defined USE_PIC24H
    #include <p24hxxxx.h>
#elif defined USE_PIC30
    #include <p30fxxxx.h>
#elif defined USE_PIC33
    #include <p33fxxxx.h>
#else
    #include <p32xxxx.h>
    #include <plib.h>
	#define Nop()	asm("nop")
#endif


#endif
